from syncloud.app import runner
from syncloud.app import logger
import logging

from subprocess import Popen, PIPE

import requests
import json
from distutils.version import StrictVersion

class Pip:

    def __init__(self, pypi_index, raise_on_error=True):
        self.logger = logger.get_logger('Pip')
        self.pypi_index = pypi_index
        self.raise_on_error = raise_on_error

    def install(self, name, version):
        if self.pypi_index:
            cmd_args = ['pip2', 'install', '--index-url', self.pypi_index, '{}=={}'.format(name, version)]
        else:
            cmd_args = ['pip2', 'install', '{}=={}'.format(name, version)]
        exit_code = runner.call(' '.join(cmd_args), self.logger, stdout_log_level=logging.INFO, shell=True)
        if self.raise_on_error and not exit_code == 0:
            raise Exception('failed command: '+' '.join(cmd_args))

    def upgrade(self, name, version):
        if self.pypi_index:
            cmd_args = ['pip2', 'install', '--index-url', self.pypi_index, '{}=={}'.format(name, version)]
        else:
            cmd_args = ['pip2', 'install', '{}=={}'.format(name, version)]
        exit_code = runner.call(' '.join(cmd_args), self.logger, stdout_log_level=logging.INFO, shell=True)
        if self.raise_on_error and not exit_code == 0:
            raise Exception('failed command: '+' '.join(cmd_args))

    def uninstall(self, name):
        cmd_args = ['yes', '|', 'pip2', 'uninstall', name]
        exit_code = runner.call(' '.join(cmd_args), self.logger, stdout_log_level=logging.INFO, shell=True)
        if self.raise_on_error and not exit_code == 0:
            raise Exception('failed command: '+' '.join(cmd_args))

    def last_version(self, name):
        if self.pypi_index:
            cmd_args = ['yolk', '--pypi-index', self.pypi_index, '--versions-available', name]
        else:
            cmd_args = ['yolk', '--versions-available', name]
        version_out = Popen(' '.join(cmd_args) , shell=True, stdout=PIPE).stdout.read().strip()
        _, version = version_out.split(' ')
        return version

    def last_version_json(self, name):
        url = "https://pypi.python.org/pypi/%s/json" % (name,)
        response = requests.get(url)
        data = json.loads(response.content)
        versions = data["releases"].keys()
        versions.sort(key=StrictVersion)
        versions.reverse()
        return versions